//*****************************************************************************************
//* Colour display for MultiWii based copters. Written by Valeriy Yatsenkov (Rover)       *
//* Free for changes and non-commercial usage.                                            *
//* Firmare contains limited code fragments from "MultiWii Data Logger"                   *
//* by Rene Schakmann: https://github.com/renes/multiwiiDataLogger                        *
//* Connect display to PORT0 crosswired as Rx->Tx, Rx->Rx, DTR->DTR, +5V->+5V, GND->GND   *
//*****************************************************************************************
//#include <SD.h>
#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library for ST7735 display chip
//#include <Adafruit_QDTech.h> // Hardware-specific library for S6D02A1 display chip
#include <SPI.h>
#include <avr/pgmspace.h>

// this string override the Arduino IDE default setting
#define SERIAL_RX_BUFFER_SIZE 128 // increased RX buffer size to receive the long data strings

#define TFT_CS     9      // display Select
#define TFT_RST    7      // display Reset (0 if use Arduino reset)
#define TFT_DC     8      // display Data/Command control line
Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS,  TFT_DC, TFT_RST); // use hardware SPI as more fast

#include "protocol.h"
#include "screenmap.h"
#include "serialstate.h"

#define INBUF_SIZE 128
#define START_PAGE 1    // start page at power on
#define TOTAL_PAGE 7    // number of pages

static uint8_t inBuf[INBUF_SIZE];
static uint8_t checksum;
static uint8_t commandMW;
static uint8_t offset;
static uint8_t dataSize;

int8_t c, page, pageTmp, GPSfixTmp, GPSsatTmp, SecTmp, Seconds, Minutes;
volatile int16_t ArmSeconds, ArmSecondsTmp;
int16_t varTmp[10]; // array for temp data values
uint32_t varTmpA, varTmpB;
int32_t varTmpC;
boolean armed, neutral, paged, armedTmp, MSPchecksum;

void setup()
{
  Serial.begin(115200);

  SPI.setClockDivider(SPI_CLOCK_DIV2);  // max possible SPI speed, 1/2 F_CLOCK

  tft.initR(INITR_BLACKTAB);     // initialize a ST7735S chip, black tab
  tft.setRotation(1);            // landscape mode (or 1 for rotation 180deg)
  tft.fillScreen(ST7735_BLACK);  // clear display to black
  tft.setTextColor(ST7735_WHITE);

  page = START_PAGE;
  paged = true;
  
  //Wait for answer of MWI
  tft.setCursor(5,60);
  tft.print("WAITING FOR CONNECTION...");
  while(commandMW != 101) {
  send_msp(MSP_STATUS, 0, 0);
  readData();
  delay(100);
  }
  tft.fillScreen(ST7735_BLACK);
}


void loop()
{
  Seconds = ((millis() + 86397000) / 1000UL) % 60;
  Minutes  = (((millis() + 86397000) / 1000UL) / 60UL) % 60;
  if (armed && (SecTmp != Seconds)) {
    ArmSeconds++;
    SecTmp = Seconds;
  }

  send_msp(MSP_RC, 0, 0);
  readData();

  if (stick_neutral() && paged) {
    neutral = true;
    paged = false;  // now ready to change page again
  }

  if (neutral && stick_page() && !paged) {
    page++;
    armedTmp = 0;
    if (page > TOTAL_PAGE) {
      page = 1;
    }
    paged = true;
  }

  // Set page order here and comment the unused case-show-break lines
  switch (page) {
    case 1: show_1(); break;//STATUS  
    case 2: show_2(); break;//RADIO
    case 3: show_3(); break;//ACTIVE MODE  
    case 4: show_4(); break;//GPS    
    case 5: show_5(); break;//MOTORS   
    case 6: show_6(); break;//SERVOS      
    case 7: show_7(); break;//SENSORS   
    default: show_1();
  }
}

// is sticks turned back from page mode to neutral position?
boolean stick_neutral() {
  uint16_t roll = readUInt16(inBuf, 0);
  uint16_t pitch = readUInt16(inBuf, 2);
  int dif1 = 1500 - roll;
  int dif2 = 1500 - pitch;
  if (abs(dif1) < 20 && abs(dif2) < 20) {
    return true;
  }
  else {
    return false;
  }
}

boolean stick_page() {
  uint16_t roll = readUInt16(inBuf, 0);
  uint16_t pitch = readUInt16(inBuf, 2);
  if (roll >= 1900 && pitch >= 1900) {
    return true;
  }
  else {
    return false;
  }
}

void send_msp(uint8_t opcode, uint8_t * data, uint8_t n_bytes) {
  uint8_t checksum = 0;
  Serial.write((byte *)"$M<", 3);
  Serial.write(n_bytes);
  checksum ^= n_bytes;
  Serial.write(opcode);
  checksum ^= opcode;
  Serial.write(checksum);
}

boolean readData() {
  delay(60);
  while (Serial.available()) {
    byte c = Serial.read();

    if (c_state == IDLE) {
      c_state = (c == '$') ? HEADER_START : IDLE;
    }
    else if (c_state == HEADER_START) {
      c_state = (c == 'M') ? HEADER_M : IDLE;
    }
    else if (c_state == HEADER_M) {
      c_state = (c == '>') ? HEADER_ARROW : IDLE;
    }
    else if (c_state == HEADER_ARROW) {

      if (c > INBUF_SIZE) {
        c_state = IDLE;
      }
      else {
        dataSize = c;
        offset = 0;
        checksum = 0;
        checksum ^= c;
        c_state = HEADER_SIZE;
      }
    }
    else if (c_state == HEADER_SIZE) {
      commandMW = c;
      checksum ^= c;
      c_state = HEADER_CMD;
    }
    else if (c_state == HEADER_CMD && offset < dataSize) {
      checksum ^= c;
      inBuf[offset++] = c;
    }
    else if (c_state == HEADER_CMD && offset >= dataSize) {
      c_state = IDLE;
    }
  }
  if (checksum == c) {
    return true;
  }
  else {
    return false;
  }
}

int16_t readInt16(uint8_t inBuf[], int index) {
  return  (inBuf[index * 2 + 1] << 8) | inBuf[index * 2];
}

uint16_t readUInt16(uint8_t inBuf[], int index) {
  return  (inBuf[index + 1] << 8) | inBuf[index];
}

uint32_t readUInt32(uint8_t inBuf[], int index) {
  uint32_t int1 = ((unsigned int)inBuf[index + 1] << 8) | inBuf[index];
  uint32_t int2 =  ((unsigned int)inBuf[index + 3] << 8) | inBuf[index + 2];
  uint32_t result = (int2 << 16) | int1;
  return result;
}

int32_t readInt32(uint8_t inBuf[], int index) {
  int32_t int1 = (inBuf[index + 1] << 8) | inBuf[index];
  int32_t int2 =  (inBuf[index + 3] << 8) | inBuf[index + 2];
  int32_t result = (int2 << 16) | int1;
  return result;
}

void checkArm() {
  send_msp(MSP_STATUS, 0, 0);
  readData();
  uint32_t boxflags = readUInt32(inBuf, 6); // parse list of flags
  armed = bitRead(boxflags, 0);
  if (armed != armedTmp) {
    tft.setCursor(ARMED_X, ARMED_Y);
    if (armed) {
      tft.setTextColor(ST7735_YELLOW);
      tft.fillRect(0, 117, 159, 10, ST7735_RED);
      tft.print("* ARMED *");
      tft.setTextColor(ST7735_WHITE);
      armedTmp = armed;
    }
    else {
      tft.fillRect(0, 117, 159, 10, ST7735_BLACK);
      armedTmp = armed;
    }
  }
}

// normalize RC data to length of bar
int norm(int j) {
  return (j - 1000) / 10 + 58;
}


























